@@ -42,17 +42,17 @@ def default_colormap():
     colormap = []
     # 1-bit colors
     colormap.append((255, 255, 255, 255))
-    colormap.append((0.0, 0.0, 0.0, 255))
+    colormap.append((0, 0, 0, 255))
     # additional 2-bit colors
     colormap.append((255, 255, 255, 255))
     colormap.append((127, 127, 127, 255))
     # additional 4-bit colors
-    colormap.append((255, 0.0, 0.0, 255))
-    colormap.append((0.0, 255, 0.0, 255))
-    colormap.append((0.0, 0.0, 255, 255))
-    colormap.append((0.0, 255, 255, 255))
-    colormap.append((255, 255, 0.0, 255))
-    colormap.append((255, 0.0, 255, 255))
+    colormap.append((255, 0, 0, 255))
+    colormap.append((0, 255, 0, 255))
+    colormap.append((0, 0, 255, 255))
+    colormap.append((0, 255, 255, 255))
+    colormap.append((255, 255, 0, 255))
+    colormap.append((255, 0, 255, 255))
     colormap.append((31, 31, 31, 255))
     colormap.append((63, 63, 63, 255))
     colormap.append((95, 95, 95, 255))
@@ -79,6 +79,8 @@ class FixedObject(object):
     """A primitive fixed-format object - eg String, Dictionary.
     value property - contains the object's value."""
     def __init__(self, value):
+        if isinstance(value, FixedObject):
+            value = value.value
         self.value = value
     
     def to_construct(self, context):
@@ -110,6 +112,9 @@ class FixedObject(object):
     
     def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__, self.value)
+    
+    def copy(self):
+        return self.__class__(self.value)
 
 
 class ContainsRefs: pass
@@ -211,6 +216,9 @@ class Collection(FixedObjectWithRepeater, ContainsRefs):
     
     def __len__(self):
         return len(self.value)
+    
+    def copy(self):
+        return self.__class__(list(self.value.copy))
 
 class Array(Collection):
     classID = 20
@@ -251,6 +259,9 @@ class Dictionary(Collection):
     
     def __getattr__(self, name):
         return getattr(self.value, name)
+    
+    def copy(self):
+        return self.__class__(self.value.copy())
 
 class IdentityDictionary(Dictionary):
     classID = 25
@@ -262,6 +273,8 @@ class IdentityDictionary(Dictionary):
 class Color(FixedObject):
     """A 32-bit RGB color value.
     Each component r, g, b has a value between 0 and 1023.
+    
+    However, Colors are considered equal if they have the same 8-bit value.
     """
     classID = 30
     _construct = BitStruct("value",
@@ -283,17 +296,39 @@ class Color(FixedObject):
         self.g = g
         self.b = b
     
+    def __eq__(self, other):
+        return (
+            isinstance(other, Color) and
+            self.to_8bit() == other.to_8bit()
+        )
+    
+    def __ne__(self, other):
+        return not self == other
+    
     def to_value(self):
         return Container(r=self.r, g=self.g, b=self.b)
     
     @classmethod
     def from_value(cls, value):
         return cls(value.r, value.g, value.b)
+    
+    @classmethod
+    def from_8bit(self, r, g=None, b=None):
+        if g is None and b is None:
+            rgb = r
+        else:
+            rgb = (r, g, b)
+        
+        return Color(*(x << 2 for x in rgb))
+        
 
     @property
     def value(self):
         return (self.r, self.g, self.b)
     
+    def copy(self):
+        return Color(self.r, self.g, self.b)
+    
     def __repr__(self):
         return "%s(%s)" % (
             self.__class__.__name__,
@@ -301,7 +336,7 @@ class Color(FixedObject):
         )
     
     def to_8bit(self):
-        """Returns value with components between 0-256."""
+        """Returns value with components between 0-255."""
         return tuple(x >> 2 for x in self.value)
     
     def to_rgba_array(self):
@@ -346,6 +381,9 @@ class TranslucentColor(Color):
     def from_value(cls, value):
         return cls(value.r, value.g, value.b, value.alpha)
     
+    def copy(self):
+        return TranslucentColor(self.r, self.g, self.b, self.alpha)
+    
     @classmethod
     def from_32bit_raw_argb(cls, raw):
         container = cls._construct_32.parse(raw)
@@ -404,6 +442,9 @@ class Point(FixedObject):
     def from_value(cls, value):
         return cls(value.x, value.y)
     
+    def copy(self):
+        return Point(self.x, self.y)
+    
     @classmethod
     def from_string(cls, string):
         (x, y) = string.split(",")
@@ -433,8 +474,7 @@ class Bitmap(FixedObjectByteArray, FixedObjectWithRepeater):
     classID = 13
     _construct = Struct("",
         UBInt32("length"),
-        construct.String("items", lambda ctx: ctx.length * 4, padchar="\x00", 
-                         paddir="right"),
+        construct.String("items", lambda ctx: ctx.length * 4),
         # Identically named "String" class -_-
     )
         
@@ -443,7 +483,10 @@ class Bitmap(FixedObjectByteArray, FixedObjectWithRepeater):
         return cls(obj.items)
 
     def to_value(self):
-        return Container(items = self.value, length = (len(self.value) + 2) / 4)
+        value = self.value
+        length = (len(value) + 3) / 4
+        value += "\x00" *  (length * 4  -  len(value)) # padding
+        return Container(items = value, length = length)
     
     _int = Struct("int",
         UBInt8("_value"),
@@ -508,6 +551,12 @@ class Bitmap(FixedObjectByteArray, FixedObjectWithRepeater):
             for pixel in run.pixels:
                 data += pixel
         return cls(data)
+    
+    
+    def compress(self):
+        """Compress to a ByteArray"""
+        raise NotImplementedError
+
 
 
 class Form(FixedObject, ContainsRefs):
@@ -552,6 +601,9 @@ class Form(FixedObject, ContainsRefs):
     def from_value(cls, value):
         return cls(**dict(value))
     
+    def copy(self):
+        return self.__class__.from_value(self.to_value())
+    
     def __repr__(self):
         return "<%s(%ix%i)>" % (
             self.__class__.__name__,
@@ -591,9 +643,9 @@ class Form(FixedObject, ContainsRefs):
         pixel_count = 0
         num_pixels = self.width * self.height
 
-        # Rows are rounded to be a whole number of words (32 bits) long.  I
-        # *think* this is because Bitmaps are run-length encoded in 32-bit
-        # segments.
+        # Rows are rounded to be a whole number of words (32 bits) long.
+        # Presumably this is because Bitmaps are compressed (run-length encoded)
+        # in 32-bit segments.
         skip = 0
         if self.depth <= 8:
             pixels_per_word = 32 / self.depth
@@ -617,25 +669,42 @@ class Form(FixedObject, ContainsRefs):
                     pixel = pixels.next()
                 x = 0
         
+        length = self.width * self.height * 4
+        blank = array("B", (0, 0, 0, 0))
+        while len(rgba) < length: 
+            rgba.extend(blank)
+        
+        assert len(rgba) == length
+        
         return (self.width, self.height, rgba)
-    
-    def save_png(self, path):
-        if not path.endswith(".png"): path += ".png"
+
+    @classmethod
+    def from_string(cls, width, height, rgba_string):
+        """Returns a Form with 32-bit RGBA pixels
+        Accepts string containing raw RGBA color values
+        """
+        # Convert RGBA string to ARGB
+        raw = ""
+        for i in range(0, len(rgba_string), 4):
+            raw += rgba_string[i+3]   # alpha
+            raw += rgba_string[i:i+3] # rgb
         
-        if not png:
-            raise ValueError, "Missing dependency: pypng library needed for " \
-                              "PNG support"
+        assert len(rgba_string) == width * height * 4
         
-        f = open(path, "wb")
-        (width, height, rgba_array) = self.to_array()
-        writer = png.Writer(width, height, alpha=True)
-        writer.write_array(f, rgba_array)
-        f.flush()
-        f.close()
+        return Form(
+            width = width,
+            height = height,
+            depth = 32,
+            bits = Bitmap(raw),
+        )
 
     @classmethod
     def from_array(cls, width, height, rgba_array):
-        """Returns a Form with 32-bit RGBA pixels"""
+        """Returns a Form with 32-bit RGBA pixels
+        Accepts sequence of flattened r, g, b, a, r, g, b, a ... values for 
+        each pixel
+        """
+        # Unused now
         raw = ""
         for i in range(0, len(rgba_array), 4):
             (r, g, b, a) = (chr(x) for x in rgba_array[i:i+4])
@@ -647,22 +716,6 @@ class Form(FixedObject, ContainsRefs):
             depth = 32,
             bits = Bitmap(raw),
         )
-    
-    @classmethod
-    def load_png(cls, path):
-        reader = png.Reader(filename=path)
-        (width, height, color_array, metadata) = reader.read_flat()
-        if metadata["bitdepth"] != 8:
-            raise ValueError("Only PNG images with depth 8 are supported")
-
-        rgba_array = array('B')
-        pixel_size = 4 if metadata["alpha"] else 3
-        for i in range(0, len(color_array), pixel_size):
-            rgba_array += color_array[i:i+pixel_size]
-            if not metadata["alpha"]:
-                rgba_array += 255
-
-        return cls.from_array(width, height, rgba_array)
 
 
 class ColorForm(Form):
